# -*- coding: utf-8 -*-
import glob, sys, os
import matplotlib.pyplot as plt
from numpy import *
import itertools

#try:
	#v1=os.path.basename(sys.argv[1])
	#v2=os.path.basename(sys.argv[2])
	#outFile=os.path.basename(sys.argv[3])
#except:
	#print "Script needs 3 arguments: version1 version2 outFileBase (e.g. 2014-01-25.git-22c2441 2014-02-24.git-b60d388 performance)"
	#sys.exit(1)

v1='2014-01-25.git-22c2441'
v2='2014-02-24.git-b60d388'
outFileBase='performance'

arch=['amd','gpu']
skiprows=1
#skiprows=5
numPoints=13

cmap = plt.get_cmap('jet')
colorList = [cmap(i) for i in linspace(0, 1, numPoints)]
#colorList=['b', 'g', 'r', 'c', 'm', 'y', 'k', '0.5']
markerList=['o', 's', 'v', 'D', '*', '^', 'p', '<', 'h', 'd', '8', '>', 'H', 'x', '+']


#### plot scaling v1/v2
fig, (ax1,ax2) = plt.subplots(ncols=2, nrows=1,figsize=(16,6))
outFile=outFileBase+'_v_scaling.jpg'
archr=arch
archr.reverse() # reverse architecture for legend

for a,ax in zip(archr,[ax1,ax2]):
	ax.set_xscale("log", nonposx='clip')
	colors = itertools.cycle(colorList)
	markers = itertools.cycle(markerList)

	for datfile1,datfile2 in zip(sorted(glob.glob(os.path.join(a,v1+"_j*.dat"))),sorted(glob.glob(os.path.join(a,v2+"_j*.dat")))):

		c = colors.next()
		m = markers.next()
		data1 = genfromtxt(datfile1,skip_header=skiprows)
		data2 = genfromtxt(datfile2,skip_header=skiprows)
		## get number of threads
		j1=datfile1[datfile1.find("_j")+2:datfile1.find(".dat")].lstrip('0')
		j2=datfile2[datfile2.find("_j")+2:datfile2.find(".dat")].lstrip('0')
		if not j1==j2:
			print datfile1+' and '+datfile2+ 'do not have the same number of cores!'
			sys.exit(1)
		ax.plot(data1[:,0], data2[:,1]/data1[:,1], color=c, linestyle='-', marker=m, label="-j"+str(j1), linewidth=2, markersize=8	)

	processor = next(open(datfile1,'r')).lstrip('# ')
	ax.set_title(processor)
	ax.set_xlabel('Number of particles')
	ax.set_ylabel('Scaling version1/version2')
	ax.set_xlim(4000,1200000)
	ax.set_ylim(0,6)
	ax.grid(True)

	#ax.legend(numpoints=1,loc='best',ncol=2)

plt.legend( loc = 'upper right', bbox_to_anchor = (0,-0.1,1,1), bbox_transform = plt.gcf().transFigure,numpoints=1, ncol=1 )
plt.savefig(outFile,bbox_inches='tight')
plt.clf()


#### compare version per architecture
for a in arch:
	fig, (ax1,ax2) = plt.subplots(ncols=2, nrows=1,figsize=(16,6))
	outFile=outFileBase+'_'+a+'.jpg'
	
	for v,ax in zip([v1,v2],[ax1,ax2]):
		ax.set_xscale("log", nonposx='clip')
		ax.set_yscale("log", nonposx='clip')
		colors = itertools.cycle(colorList)
		markers = itertools.cycle(markerList)

		for datfile in sorted(glob.glob(os.path.join(a,v+"_j*.dat"))):
			c = colors.next()
			mk = markers.next()
			data = genfromtxt(datfile,skip_header=skiprows)
			## get number of threads
			n=datfile.find("_j")
			m=datfile.find(".dat")
			j=datfile[n+2:m].lstrip('0')
			ax.plot(data[:,0], data[:,1], color=c, linestyle='-', label="-j"+str(j), marker=mk, linewidth=2)
			ax.errorbar(data[:,0], data[:,1], yerr=data[:,2], color=c, linestyle='', markersize=8)

		archLabel = 'AMD' if a == 'amd' else 'Intel'
		ax.set_title(v+' ('+archLabel+')\n')
		ax.set_xlabel('Number of particles')
		ax.set_ylabel('Iterations per second')
		ax.grid(True)
		ax.set_xlim(4000,1200000)
		ax.set_ylim(0.1,1000)
		#ax.legend(numpoints=1,loc='best',ncol=3)

	plt.legend( loc = 'upper right', bbox_to_anchor = (0,-0.1,1,1), bbox_transform = plt.gcf().transFigure,numpoints=1, ncol=1 )
	plt.savefig(outFile,bbox_inches='tight')
	plt.clf()


#### compare Cundall number per architecture
for a in arch:
	fig, (ax1,ax2) = plt.subplots(ncols=2, nrows=1,figsize=(16,6))
	outFile=outFileBase+'_'+a+'_cundall.jpg'
	
	for v,ax in zip([v1,v2],[ax1,ax2]):
		ax.set_xscale("log", nonposx='clip')
		ax.set_yscale("log", nonposx='clip')
		colors = itertools.cycle(colorList)
		markers = itertools.cycle(markerList)

		for datfile in sorted(glob.glob(os.path.join(a,v+"_j*.dat"))):
			c = colors.next()
			mk = markers.next()
			data = genfromtxt(datfile,skip_header=skiprows)
			## get number of threads
			n=datfile.find("_j")
			m=datfile.find(".dat")
			j=datfile[n+2:m].lstrip('0')
			ax.plot(data[:,0], data[:,1]*data[:,0], color=c, linestyle='-', label="-j"+str(j), marker=mk, linewidth=2)
			ax.errorbar(data[:,0], data[:,1]*data[:,0], yerr=data[:,2], color=c, linestyle='', markersize=8)

		archLabel = 'AMD' if a == 'amd' else 'Intel'
		ax.set_title(v+' ('+archLabel+')\n')
		ax.set_xlabel('Number of particles')
		ax.set_ylabel('Cundall number')
		ax.grid(True)
		ax.set_xlim(4000,1200000)
		ax.set_ylim(100000,3000000)
		#ax.legend(numpoints=1,loc='best',ncol=3)

	plt.legend( loc = 'upper right', bbox_to_anchor = (0,-0.1,1,1), bbox_transform = plt.gcf().transFigure,numpoints=1, ncol=1 )
	plt.savefig(outFile,bbox_inches='tight')
	plt.clf()



#### compare architecture
fig, (ax1,ax2) = plt.subplots(ncols=2, nrows=1,figsize=(16,6))
outFile=outFileBase+'_arch_scaling.jpg'

for v,ax in zip([v1,v2],[ax1,ax2]):
	ax.set_xscale("log", nonposx='clip')
	colors = itertools.cycle(colorList)
	markers = itertools.cycle(markerList)

	for datfile1,datfile2 in zip(sorted(glob.glob(os.path.join(arch[0],v+"_j*.dat"))),sorted(glob.glob(os.path.join(arch[1],v+"_j*.dat")))):

		c = colors.next()
		m = markers.next()
		data1 = genfromtxt(datfile1,skip_header=skiprows)
		data2 = genfromtxt(datfile2,skip_header=skiprows)
		## get number of threads
		j1=datfile1[datfile1.find("_j")+2:datfile1.find(".dat")].lstrip('0')
		j2=datfile2[datfile2.find("_j")+2:datfile2.find(".dat")].lstrip('0')
		if not j1==j2:
			print datfile1+' and '+datfile2+ 'do not have the same number of cores!'
			sys.exit(1)
		ax.plot(data1[:,0], data1[:,1]/data2[:,1], color=c, linestyle='-', marker=m, label="-j"+str(j1), linewidth=2, markersize=8)

	ax.set_title(v+'\n')
	ax.set_xlabel('Number of particles')
	ax.set_ylabel('Scaling AMD/Intel')
	ax.set_xlim(4000,1200000)
	ax.set_ylim(1,5)
	ax.grid(True)
	#ax.legend(numpoints=1,loc='best',ncol=3)

plt.legend( loc = 'upper right', bbox_to_anchor = (0,-0.1,1,1), bbox_transform = plt.gcf().transFigure,numpoints=1, ncol=1 )
#plt.legend( loc = 'lower center', bbox_to_anchor = (0,-0.1,1,1), bbox_transform = plt.gcf().transFigure,numpoints=1, ncol=12 )
plt.savefig(outFile,bbox_inches='tight')
plt.clf()

